@@ -451,7 +451,7 @@ class Node(Element, Observable):
     def __init__(self, **attributes):
         '''
         Initialize node attributes
-        
+
         @param attributes: dictionary of initialization attributes
         @type attributes: dict
         '''
@@ -672,17 +672,17 @@ class Node(Element, Observable):
     def removeChild(self, child):
         '''
         Remove a child node from the current node
-        
+
         @type child: Node
         '''
+        self.notifyObservers(child, type = 'childNodeRemoved')
         self.__children.remove(child)
         child.__parent = None
-        self.notifyObservers(child, type = 'childNodeRemoved')
 
     def getChildPosition(self, child):
         '''
         Return position of a child in the current node
-        
+
         @type child: Node
         '''
         if not self.isAncestorOf(child):
@@ -745,7 +745,7 @@ class Node(Element, Observable):
         while node.parent is not None:
             nodeDepth += 1
             node = node.parent
-        return nodeDepth
+        return nodeDepth * (-1 if self.left else 1)
 
     def createChildPrototype(self, **attributes):
         '''
@@ -753,7 +753,7 @@ class Node(Element, Observable):
         Attributes of the created node are set according to the inheritance policy.
         Prototype is not a child of this node yet. You must add it afterwards with the
         addChilde method
-        
+
         @param attributes: initialization attributes
         @type attributes: dict
 
@@ -783,9 +783,9 @@ class Node(Element, Observable):
     def addNodeAttribute(self, attrName, attrValue):
         '''
         Add attribute to the current node
-        
+
         @type attrName: str
-        @type attrValue: str    
+        @type attrValue: str
         '''
         if self.__nodeAttribute is None:
             self.__nodeAttribute = Attribute()
@@ -794,7 +794,7 @@ class Node(Element, Observable):
     def paint(self, painter):
         '''
         Paint node text content
-        
+
         @type painter: QPainter
         '''
         painter.save()
@@ -806,7 +806,7 @@ class Node(Element, Observable):
     def textSize(self):
         '''
         Return area size of node text content
-        
+
         @rtype: QSizeF
         '''
         size = self.__richcontent.size()
@@ -863,7 +863,7 @@ class Cloud(Element):
     def __init__(self, **attributes):
         '''
         Initialize cloud attributes
-        
+
         @param attributes: dictionary of initialization attributes
         @type attributes: dict
         '''
@@ -884,16 +884,16 @@ class AttributeRegistry(Element):
     def __init__(self, **attributes):
         '''
         Initialize empty attribute registry
-        
+
         @param attributes: dictionary of initialization attributes
         @type attributes: dict
         '''
         Element.__init__(self, **attributes)
-        # dict of attributes options in following format: 
+        # dict of attributes options in following format:
         # {'name_A' : {'option_x' : 'value_x', 'option_y' : 'value_y'}
         #  'name_B' : {'option_z' : 'value_z'} ...}
         self.__viewOptions = {}
-        # dict of attributes values in following format: 
+        # dict of attributes values in following format:
         # {'name_A' : ['value_1', 'value_2'], 'name_B' : ['value_1'] ...}
         self.__attrValues = {}
 
@@ -912,21 +912,21 @@ class Attribute(Element):
     def __init__(self, **attributes):
         '''
         Initialize class attributes
-        
+
         @param attributes: dictionary of initialization attributes
         @type attributes: dict
         '''
         Element.__init__(self, **attributes)
-        # dict of attributes in following format: 
+        # dict of attributes in following format:
         # {'name_A' : 'value_A', 'name_B' : 'value_B' ...}
         self.__allAttributes = {}
 
     def addAttribute(self, attrName, attrValue):
         '''
         Add attribute to the container
-        
+
         @type attrName: str
-        @type attrValue: str    
+        @type attrValue: str
         '''
         self.__allAttributes[attrName] = attrValue
 
