@@ -26,8 +26,9 @@ Dialogs for Desktop platform
 
 from PyQt4.QtGui import QToolBar, QIcon, QTabWidget, QGridLayout, \
 QHBoxLayout, QPalette, QTextBrowser, QFrame, QLabel, QDialogButtonBox, \
-QVBoxLayout, QGroupBox, QWidget, QPushButton, QKeySequence
+QVBoxLayout, QGroupBox, QWidget, QPushButton, QKeySequence, QPixmap
 from PyQt4.QtCore import QSize, Qt
+from hivemind import resource
 from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
 AboutDialog, EditEdgeDialog, SettingsDialog, NetworkConnectionDialog, \
 AffiliationDialog, NetworkSettingsDialog, NetworkClientDialog
@@ -352,12 +353,11 @@ class DesktopFilterToolBar(HivemindToolBar, FilterToolBar):
     Filter tool bar for Desktop platform
     @author: Oleg Kandaurov
     '''
-    def __init__(self, controller, parent = None):
+    def __init__(self, mindMapScene, parent = None):
         '''
-        @type controller: MainWindowController
         @type parent: QWidget
         '''
-        FilterToolBar.__init__(self, controller)
+        FilterToolBar.__init__(self, mindMapScene)
         HivemindToolBar.__init__(self, parent)
         self.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
         self._createComponents()
@@ -366,9 +366,14 @@ class DesktopFilterToolBar(HivemindToolBar, FilterToolBar):
     def _createComponents(self):
         '''Create components'''
         #pylint: disable=W0201
-        self._modeAction = StatusAction('Filtration mode', 'filtermode', 'text', self, 32)
+        self._modeAction = StatusAction('Filtration mode', 'filtermode', 'text', self, 24)
         FilterToolBar._createComponents(self)
 
+    @classmethod
+    def _getPixmapByIconName(cls, icon):
+        '''Return pixmap for icon name'''
+        return QPixmap.fromImage(resource.getIconByName(icon)).scaled(QSize(24, 24))
+
     def _initComponents(self):
         '''Init components'''
         self._closeButton.clicked.connect(self._close)
