@@ -34,7 +34,7 @@ from PyQt4.QtCore import SIGNAL, QRegExp, Qt, QSize, QCoreApplication, QObject,
 from hivemind.attribute import readable, adaptCallable
 from hivemind.gui_widgets import SearchingTrMixin, SpecialTextEdit, SettingsCreator, \
 ScalingWidget, IconList, IconTable, LineEditBinder, ColorLabel
-from hivemind.network.models import AffiliationDelegate
+from hivemind.network.models import AffiliationDelegate, DeleteButton
 from twisted.words.protocols.jabber import jid
 from hivemind.smartconference.kp import SmartSpaceChecker
 
@@ -1058,6 +1058,7 @@ class AffiliationDialog(CommonDialog):
         self.__model = model
         self.__rosterModel = rosterModel
         self.setWindowTitle(self.tr('Permissions'))
+        self._buttonBox.button(QDialogButtonBox.Cancel).setVisible(False)
         self._createComponents()
         self._initComponents()
 
@@ -1065,13 +1066,19 @@ class AffiliationDialog(CommonDialog):
         '''Create components'''
         #pylint: disable = W0201
         self.__delegate = AffiliationDelegate()
+        self.__delDelegate = DeleteButton()
         self.__view = QTableView()
         self.__view.setModel(self.__model)
         self.__view.setItemDelegateForColumn(1, self.__delegate)
+        self.__view.setItemDelegateForColumn(3, self.__delDelegate)
         self.__view.setEditTriggers(QAbstractItemView.AllEditTriggers)
+        verticalHeader = self.__view.verticalHeader()
         horizontalHeader = self.__view.horizontalHeader()
-        horizontalHeader.setResizeMode(QHeaderView.Stretch)
-        horizontalHeader.setStretchLastSection(True)
+        horizontalHeader.setResizeMode(0, QHeaderView.Stretch)
+        horizontalHeader.setResizeMode(1, QHeaderView.Stretch)
+        horizontalHeader.setResizeMode(2, QHeaderView.Stretch)
+        horizontalHeader.resizeSection(3, verticalHeader.defaultSectionSize())
+        horizontalHeader.setResizeMode(3, QHeaderView.Fixed)
         self.__jidLabel = QLabel(self.tr('JID'))
         self.__jidEdit = QLineEdit()
         self.__rosterCombo = QComboBox()
@@ -1084,6 +1091,7 @@ class AffiliationDialog(CommonDialog):
         self.__additionButton = QPushButton(self.tr('Add participant'))
         self.__additionButton.clicked.connect(self._addParticipant)
         self.__rosterCombo.activated.connect(self._itemClicked)
+        self.__view.clicked.connect(self._viewItemClicked)
 
     def _initComponents(self):
         '''Initialize components'''
@@ -1114,6 +1122,11 @@ class AffiliationDialog(CommonDialog):
         mainLayout.addWidget(self._buttonBox)
         self.setLayout(mainLayout)
 
+    def _viewItemClicked(self, index):
+        '''Process clicked signal from view'''
+        if index.column() == 3:
+            self.__model.deleteParticipant(index.row())
+
     def _itemClicked(self, index):
         '''Process clicked signal from rosterCombo'''
         modelIndex = self.__rosterModel.index(index, 0)
@@ -1125,7 +1138,7 @@ class AffiliationDialog(CommonDialog):
         if not jidRegexp.exactMatch(self.__jidEdit.text()):
             _showJIDErrorDialog(self)
             return
-        self.__model.updateAffiliation(jid.JID(unicode(self.__jidEdit.text())).userhost(),
+        self.__model.addParticipant(jid.JID(unicode(self.__jidEdit.text())).userhost(),
                 unicode(self.__affiliationCombo.itemData(
                         self.__affiliationCombo.currentIndex()).toString()))
         self.__jidEdit.clear()
