@@ -135,16 +135,29 @@ class MindMapDelegate(QGraphicsScene):
         for child in root.children:
             self._addDelegatesForNodeSubtree(child)
 
+    def calculateVisibilty(self, node, propagate):
+        '''
+        Calculates visibility for given node and optionally for it children
+        @param node: Given node
+        @param propagate: Propagate calculation to children nodes
+        '''
+        visible = False if node.folded is not None else True
+        for child in node.children:
+            if not self.__nodeToDelegate[child].matched:
+                self.__nodeToDelegate[child].setVisible(False)
+                continue
+            if self.__nodeToDelegate[child].isVisible() != visible:
+                self.__nodeToDelegate[child].setVisible(visible)
+            if propagate:
+                self.calculateVisibilty(child, propagate)
+
     def nodeBoundsChanged(self, node):
         '''
         This method called, then signal for node bounds changed has come
         @param node: modified node
         @type node: Node
         '''
-        visible = False if node.folded is not None else True
-        for child in node.children:
-            if self.__nodeToDelegate[child].isVisible() != visible:
-                self.__nodeToDelegate[child].setVisible(visible)
+        self.calculateVisibilty(node, propagate = False)
         self.__nodeToDelegate[node].updateNodeBounds(False)
         self.positionNodes(node)
 
@@ -174,7 +187,7 @@ class MindMapDelegate(QGraphicsScene):
         '''
         delegate = self.__nodeToDelegate[node]
         if node.isAncestorOf(self.__currentNode):
-            self.__nodeToDelegate[node.parent].setLastChild(None)
+            self.__nodeToDelegate[node.parent].setLastChild(None, node.left)
             self.currentNode = node.parent
         self._removeNodeDelegateSubtree(node)
         self.removeItem(delegate)
@@ -199,8 +212,10 @@ class MindMapDelegate(QGraphicsScene):
         @param dest: destination node
         @type dest: Node
         '''
-        parentDelegate = self.__nodeToDelegate[dest]
         roamingDelegate = self.__nodeToDelegate[node]
+        previousParentDelegate = roamingDelegate.parentItem()
+        previousParentDelegate.setLastChild(None, not node.left)
+        parentDelegate = self.__nodeToDelegate[dest]
         roamingDelegate.setParentItem(parentDelegate)
         if dest.folded: roamingDelegate.hide()
         # node relocation require recalculate position of all nodes
@@ -231,6 +246,13 @@ class MindMapDelegate(QGraphicsScene):
             self.views()[0].setViewportUpdateMode(QGraphicsView.SmartViewportUpdate)
         self.mapChanged.emit()
 
+    def getFilteredMindMap(self):
+        '''
+        Get filtered mind map
+        @rtype: MindMap
+        '''
+        return MindMap(self.__rootDelegate.getFilteredTree())
+
     _MAP_MARGIN_WIDTH = 300
     ''' Width of mind map margin '''
     _MAP_MARGIN_HEIGHT = 150
@@ -462,7 +484,7 @@ class MindMapDelegate(QGraphicsScene):
         '''
         if self.currentNode.left != left and self.currentNode.parent is not None:
             parentNodeDelegate = self.__nodeToDelegate[self.currentNode.parent]
-            parentNodeDelegate.setLastChild(self.currentNode)
+            parentNodeDelegate.setLastChild(self.currentNode, self.currentNode.left)
             self.currentNode = self.currentNode.parent
             return
         if not self.currentNode.children or self.currentNode.folded:
@@ -471,9 +493,9 @@ class MindMapDelegate(QGraphicsScene):
         if lastChild:
             self.currentNode = lastChild
             return
-        self._selectNearestChidlNode(left)
+        self._selectNearestChildNode(left)
 
-    def _selectNearestChidlNode(self, left):
+    def _selectNearestChildNode(self, left):
         '''
         Select nearest child node of current selected node
         @param left: if true search node in left subtree, search in right subtree otherwise
@@ -493,12 +515,27 @@ class MindMapDelegate(QGraphicsScene):
         @param up: if True select node above otherwise below
         '''
         nodeMap = self.__nodeMapCreator.createNodeMap(self.rootDelegate.node)
-        nodes = nodeMap[self.currentNode.depth()]
-        newPos = nodes.index(self.currentNode) + (-1 if up else 1)
-        if newPos in range(0, len(nodes)):
-            nextNode = nodes[newPos]
-            if self.getDelegate(nextNode).isVisible():
-                self.currentNode = nextNode
+        depth = self.currentNode.depth()
+        nodes = nodeMap[depth]
+        pos = nodes.index(self.currentNode)
+        prevNode = self.currentNode
+        movement = -1 if up else 1
+        reduceDepth = 1 if prevNode.left else -1
+        pos += movement
+        while True :
+            if pos not in range(0, len(nodes)):
+                if depth == 0 or abs(depth) == 1:
+                    return
+                depth += reduceDepth
+                nodes = nodeMap[depth]
+                pos = nodes.index(prevNode.parent) + movement
+                prevNode = prevNode.parent
+                continue
+            if self.getDelegate(nodes[pos]).isVisible():
+                self.currentNode = nodes[pos]
+                return
+            prevNode = nodes[pos]
+            pos += movement
 
 
 class NodeDelegate(QGraphicsItem):
@@ -537,6 +574,19 @@ class NodeDelegate(QGraphicsItem):
 
     highlighted = property(_isHighlighted, _setHighlighted, None, 'Highlighted flag')
 
+    def _isMatched(self):
+        '''Get filtered flag'''
+        return self.__matched
+
+    def _setMatched(self, matched):
+        '''
+        Set the filtered flag
+        @type matched: boolean
+        '''
+        self.__matched = matched
+
+    matched = property(_isMatched, _setMatched, None, 'Matched flag')
+
     def __init__(self, node, parent = None):
         '''
         Initialize delegate with specified node
@@ -549,6 +599,7 @@ class NodeDelegate(QGraphicsItem):
         self.__node = node
         self.__verticalSpace = 0
         self.__highlighted = False
+        self.__matched = True
         self.__selected = False
         self.__positionChanged = True
         self.__textBoundingRect = None
@@ -616,6 +667,16 @@ class NodeDelegate(QGraphicsItem):
         '''Get all child node delegates of the current delegate, filter out other elements'''
         return [item for item in self.childItems() if isinstance(item, NodeDelegate)]
 
+    def getFilteredTree(self):
+        '''
+        Create a clone of the current node and it's filtered subtree
+        @rtype: Node
+        '''
+        root = self.node.clone()
+        for child in self._childDelegates():
+            if child.matched: root.addChild(child.getFilteredTree())
+        return root
+
     def updateSubtree(self):
         '''
         Update view of the delegate and it subtree
@@ -1028,14 +1089,16 @@ class NodeDelegate(QGraphicsItem):
         if lastChild not in self.__node.children: lastChild = None
         return lastChild
 
-    def setLastChild(self, child):
+    def setLastChild(self, child, left):
         '''
         Set this node as a last child
         @param child: child node that was selected before this node
         @type child: Node
+        @param left: indicator that child is in left subtree
+        @type left: bool
         '''
         if self.__node.parent is None:
-            if child.left:
+            if left:
                 self.__lastChild[0] = child
             else:
                 self.__lastChild[1] = child
@@ -1313,21 +1376,20 @@ class MindMapFilter(QObject):
 
     writable('mode', 'text', 'icons')
 
-    def __init__(self, controller):
+    def __init__(self, mindMapScene):
         '''Constructor'''
         QObject.__init__(self)
-        self.__controller = controller
-        self.__mindMapScene = controller.mindMapScene
+        self.__mindMapScene = mindMapScene
         self.__nodemapCreator = self.__mindMapScene.nodeMapCreator
         self.__foundNodes = []
-        self.__filterMode = False
+        self.__isfilterModeEnabled = False
         self.__text = None
         self.__icons = []
         self.__mode = None
         self.__mindMapScene.curMapChanged.connect(self.refresh)
         self.__mindMapScene.mapChanged.connect(self.refresh)
 
-    def _textCondition(self, node):
+    def _isMatchTextCondition(self, node):
         '''
         Decide whether node match a condition or not
         @param node: Node to be inspected
@@ -1341,7 +1403,7 @@ class MindMapFilter(QObject):
             return True
         return False
 
-    def _iconCondition(self, node):
+    def _isMatchIconCondition(self, node):
         '''
         Decide whether node match a condition or not
         @param node: Node to be inspected
@@ -1354,58 +1416,39 @@ class MindMapFilter(QObject):
                 return True
         return False
 
-    def _filter(self, node, isVisible):
+    def _setMatchedProperty(self, node):
         '''
-        Filter given node
-        @param node: Node that need to be examined
-        @param isVisible: False if the node ancestor is folded
+        Traverses through all children nodes and sets "matched" property
+        @param node: Parent node
         '''
-        nodeDelegate = self.__mindMapScene.getDelegate(node)
-        if nodeDelegate is None: return
-        if node in self.__foundNodes and self.__filterMode:
-            self.__controller.showNode(node)
-            nodeDelegate.setVisible(True)
-        elif isVisible:
-            nodeDelegate.setVisible(not self.__filterMode)
-
-    def _filterNodes(self):
-        '''Apply filter to all nodes'''
-        def traverse(node, isVisible):
-            '''
-            Traverse through children nodes
-            @param node: Parent node
-            @param isVisible: False if the node ancestor is folded
-            '''
-            for node in node.children:
-                if isVisible: isVisible = node.folded is None
-                self._filter(node, isVisible)
-                traverse(node, isVisible)
-
-        traverse(self.__mindMapScene.rootDelegate.node, True)
-
-    def toggleFilter(self, filterMode):
-        '''
-        Update filter trigger
-        @param filterMode: Indicates that filtering is enabled
-        '''
-        self.__filterMode = filterMode
-        if not filterMode:
-            self._filterNodes()
-        else:
-            self.refresh()
+        for node in node.children:
+            nodeDelegate = self.__mindMapScene.getDelegate(node)
+            if nodeDelegate is None: return
+            nodeDelegate.matched = ((node in self.__foundNodes)
+                    if self.__isfilterModeEnabled else True)
+            self._setMatchedProperty(node)
 
     def refresh(self):
-        '''Find nodes and filter it'''
-        nodeMap = self.__nodemapCreator.createNodeMap(self.__mindMapScene.rootDelegate.node)
-        if self.__mode == 'text':
-            filteredNodeMap = self.__nodemapCreator.createFilteredNodeMap(nodeMap,
-                    self._textCondition)
-        else:
-            filteredNodeMap = self.__nodemapCreator.createFilteredNodeMap(nodeMap,
-                    self._iconCondition)
-        self.__nodemapCreator.includeParentNodes(filteredNodeMap)
-        self.__foundNodes = self.__nodemapCreator.createSortedNodeList(filteredNodeMap)
-        self._filterNodes()
+        '''Apply filter to all nodes'''
+        if self.__isfilterModeEnabled:
+            condition = (self._isMatchTextCondition if self.__mode == 'text'
+                    else self._isMatchIconCondition)
+            nodeMap = self.__nodemapCreator.createNodeMap(
+                    self.__mindMapScene.rootDelegate.node)
+            filteredNodeMap = self.__nodemapCreator.createFilteredNodeMap(nodeMap, condition)
+            self.__nodemapCreator.includeParentNodes(filteredNodeMap)
+            self.__foundNodes = self.__nodemapCreator.createSortedNodeList(filteredNodeMap)
+        self._setMatchedProperty(self.__mindMapScene.rootDelegate.node)
+        self.__mindMapScene.calculateVisibilty(self.__mindMapScene.rootDelegate.node,
+                propagate = True)
+
+    def toggleFilter(self, isfilterModeEnabled):
+        '''
+        Update filter trigger
+        @param isfilterModeEnabled: Indicates that filtering is enabled
+        '''
+        self.__isfilterModeEnabled = isfilterModeEnabled
+        self.refresh()
 
 
 class NodeFinder(QObject):
