@@ -127,14 +127,14 @@ def createFindToolBar(controller):
         return MaemoFindToolBar(controller, __defaultParent)
     return DesktopFindToolBar(controller, __defaultParent)
 
-def createFilterToolBar(controller):
+def createFilterToolBar(mindMapScene):
     '''
     Factory for creating families of related filter toolbars
     @type controller: MainWindowController
     '''
     if sysconfig.PLATFORM == 'maemo':
-        return MaemoFilterToolBar(controller, __defaultParent)
-    return DesktopFilterToolBar(controller, __defaultParent)
+        return MaemoFilterToolBar(mindMapScene, __defaultParent)
+    return DesktopFilterToolBar(mindMapScene, __defaultParent)
 
 def createSettingsDialog():
     '''
