@@ -41,7 +41,7 @@ from PyQt4.QtGui import QActionGroup, QGraphicsBlurEffect, QMessageBox, QUndoSta
 QFileDialog, QPixmap, QPainter, QAction, QDesktopServices, QPrintDialog, QPrinter, \
 QUndoCommand, QProgressDialog
 from PyQt4.QtCore import QObject, SIGNAL, QTimer, QLocale, QTranslator, QLibraryInfo, \
-Qt, QRectF, QUrl
+Qt, QRectF, QUrl, QDir
 import hivemind.gui_factory as gui_factory
 import hivemind.parser as parser
 import hivemind.settings as settings
@@ -107,11 +107,13 @@ class MainWindowController(QObject):
         self.__mainWindow.show()
         _configureApplication(self.__mindMapController)
         self.__findWidget = gui_factory.createFindToolBar(self)
-        self.__filterWidget = gui_factory.createFilterToolBar(self)
+        self.__filterWidget = gui_factory.createFilterToolBar(self.__mindMapScene)
         self.__findWidget.setObjectName('findToolbar')
         self.__filterWidget.setObjectName('filterToolbar')
         self._addToolbar(self.__filterWidget)
         self._addToolbar(self.__findWidget)
+        QObject.connect(self.__filterWidget, SIGNAL('visibilityChanged(bool)'),
+                self.__actionBag.fileSaveFilterAction.setVisible)
 
     def _addToolbar(self, toolbar):
         '''
@@ -450,12 +452,16 @@ class MainWindowController(QObject):
         self.__filterWidget.setVisible(False)
         self.__findWidget.setDefaultFocus()
         self.__findWidget.setVisible(not self.__findWidget.isVisible())
+        self.__actionBag.selectRootNodeAction.setDisabled(
+                self.__findWidget.isVisible() or self.__filterWidget.isVisible())
 
     def onFilter(self):
         '''Process filter action'''
         self.__findWidget.setVisible(False)
         self.__filterWidget.setVisible(not self.__filterWidget.isVisible())
         self.__filterWidget.enableFiltration(self.__filterWidget.isVisible())
+        self.__actionBag.selectRootNodeAction.setDisabled(
+                self.__findWidget.isVisible() or self.__filterWidget.isVisible())
 
     def editEdge(self):
         '''Edit node's edge decorations and style'''
@@ -617,6 +623,7 @@ class MindMapController(QObject):
         self.__actionBag.fileOpenAction.triggered.connect(self.onFileOpen)
         self.__actionBag.fileSaveAction.triggered.connect(self.onFileSave)
         self.__actionBag.fileSaveAsAction.triggered.connect(self.onFileSaveAs)
+        self.__actionBag.fileSaveFilterAction.triggered.connect(self.onFileSaveFilter)
         self.__actionBag.exportToImageAction.triggered.connect(self.onExportToImage)
         self.__actionBag.printAction.triggered.connect(self.onPrint)
         for action in self.__actionBag.recentFileActions:
@@ -769,6 +776,24 @@ class MindMapController(QObject):
                     unicode(args[1]))
             return False
 
+    def onFileSaveFilter(self):
+        '''
+        Process file save filtered map action
+        @return: True if mind map was saved False otherwise
+        @rtype: bool
+        '''
+        fileName = self._getSaveFileName(self.tr('Save as'),
+                [[self.tr('Mind maps (*.mm)'), '.*\.mm', '.mm']])
+        if not fileName: return False
+        try:
+            filteredMindMap = self.__mindMapScene.getFilteredMindMap()
+            parser.storeMindMap(fileName, filteredMindMap)
+            return True
+        except IOError, args:
+            QMessageBox.critical(gui_factory.defaultParent(), self.tr('File save error'),
+                    unicode(args[1]))
+            return False
+
     def deserializeData(self, data, dataType):
         '''
         Deserialize data according to the type
@@ -807,6 +832,7 @@ class MindMapController(QObject):
         @type clean: bool
         '''
         self.__actionBag.fileSaveAction.setDisabled(clean)
+        self._updateWindowTitle()
 
     def _showSaveChangesBox(self):
         '''
@@ -847,8 +873,15 @@ class MindMapController(QObject):
     def _setFileName(self, fileName):
         '''Set __filename field and window header'''
         self.__fileName = fileName
-        if fileName is not None:
-            gui_factory.defaultParent().setWindowTitle(os.path.split(fileName)[1])
+        self._updateWindowTitle()
+
+    def _updateWindowTitle(self):
+        '''Update title of the main window according to undo stack state'''
+        if self.__fileName is not None:
+            name = os.path.split(self.__fileName)[1]
+            if not self.__undoStack.isClean():
+                name = name + '*'
+            gui_factory.defaultParent().setWindowTitle(name)
         else:
             gui_factory.defaultParent().setWindowTitle(None)
 
@@ -868,6 +901,11 @@ class MindMapController(QObject):
         '''
         if path is not None:
             dirname, filename = os.path.split(path)
+            if not os.access(dirname, os.W_OK):
+                tempPath = os.path.join(unicode(QDir.tempPath()), 'hivemind')
+                if not os.access(tempPath, os.F_OK):
+                    os.mkdir(tempPath)
+                return os.path.join(tempPath, '.#' + filename + str(hash(path)) + '#')
             return os.path.join(dirname, '.#' + filename + '#')
         return None
 
