@@ -1933,13 +1933,11 @@ class FilterToolBar(SearchingTrMixin, QObject):
     Abstract class for filter toolbar
     @author: Oleg Kandaurov
     '''
-    def __init__(self, controller):
-        '''
-        @type controller: MainWindowController
-        '''
+    def __init__(self, mindMapScene):
+        '''Constructor'''
         QObject.__init__(self)
         SearchingTrMixin.__init__(self)
-        self._filter = MindMapFilter(controller)
+        self._filter = MindMapFilter(mindMapScene)
         self._closeButton = QPushButton()
         self.__node = Node()
         self._iconDialog = hivemind.gui_factory.createIconDialog(self.__node)
@@ -2040,7 +2038,7 @@ class FilterToolBar(SearchingTrMixin, QObject):
         labels = []
         for icon in self.__node.icons:
             label = QLabel()
-            label.setPixmap(QPixmap.fromImage(resource.getIconByName(icon)))
+            label.setPixmap(self._getPixmapByIconName(icon))
             labels.append(label)
         return labels
 
