@@ -27,14 +27,15 @@ Dialogs for Maemo platform
 import hivemind.resource as resource
 from PyQt4.QtGui import QFrame, QWidget, QGridLayout, QStackedWidget, QVBoxLayout, \
     QRadioButton, QHBoxLayout, QToolBar, QIcon, QPalette, QPushButton, QLabel, \
-    QTextBrowser, QDialog, QFont, QDesktopServices, QAction
+    QTextBrowser, QDialog, QFont, QDesktopServices, QAction, QPixmap
 from PyQt4.QtCore import Qt, QSize, QTimer, QUrl
 from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
     AboutDialog, EditEdgeDialog, SettingsDialog, AffiliationDialog, NetworkSettingsDialog, \
-    NetworkClientDialog, NetworkConnectionDialog
+    NetworkClientDialog, NetworkConnectionDialog, CommonDialog
 from hivemind.gui_widgets import ScalingWidget, ActionImageButton, FindToolBar, \
     FilterToolBar, MaemoToolBar, StatusAction
 from hivemind.attribute import adaptCallable
+from hivemind import settings
 import hivemind.__version__ as __version__
 
 
@@ -512,6 +513,7 @@ class MaemoNetworkMainDialog(MaemoDialogMixin, QDialog):
         self.__startClientButton = QPushButton(self.__bag.startClientAction.text())
         self.__networkSettingsButton = QPushButton(self.__bag.networkSettingsAction.text())
         self.__networkHelpButton = QPushButton(self.__bag.networkHelpAction.text())
+        self.__helpDialog = MaemoTeamworkHelpDialog(self)
 
     def _initComponents(self):
         '''Init components'''
@@ -543,6 +545,9 @@ class MaemoNetworkMainDialog(MaemoDialogMixin, QDialog):
         self.__bag.startClientAction.changed.connect(self._updateState)
         self.__bag.networkHelpAction.changed.connect(self._updateState)
         self.__bag.networkSettingsAction.changed.connect(self._updateState)
+        self.__stopProtocolButton.clicked.connect(self.close)
+        self.__startServiceButton.clicked.connect(self.close)
+        self.__bag.networkHelpAction.triggered.connect(self._showHelpDialog)
 
     def _updateState(self):
         '''Update button states acccording to action state'''
@@ -554,18 +559,27 @@ class MaemoNetworkMainDialog(MaemoDialogMixin, QDialog):
         self.__networkHelpButton.setEnabled(self.__bag.networkHelpAction.isEnabled())
         self.__networkSettingsButton.setEnabled(self.__bag.networkSettingsAction.isEnabled())
 
+    def _showHelpDialog(self):
+        '''Show help dialog'''
+        self.__helpDialog.exec_()
+
+    def exec_(self):
+        '''Show teamwork mode'''
+        MaemoDialogMixin.exec_(self)
+        if settings.get('teamworkModeFirstRun'):
+            settings.set(teamworkModeFirstRun = False)
+            self.__helpDialog.exec_()
 
 class MaemoFilterToolBar(MaemoToolBar, FilterToolBar):
     '''
     Filter tool bar for Maemo platform
     @author: Oleg Kandaurov
     '''
-    def __init__(self, controller, parent = None):
+    def __init__(self, mindMapScene, parent = None):
         '''
-        @type controller: MainWindowController
         @type parent: QWidget
         '''
-        FilterToolBar.__init__(self, controller)
+        FilterToolBar.__init__(self, mindMapScene)
         MaemoToolBar.__init__(self, parent)
         self._createComponents()
         self._initComponents()
@@ -576,6 +590,11 @@ class MaemoFilterToolBar(MaemoToolBar, FilterToolBar):
         self._modeAction = StatusAction('Filtration mode', 'filtermode', 'text')
         FilterToolBar._createComponents(self)
 
+    @classmethod
+    def _getPixmapByIconName(cls, icon):
+        '''Return pixmap for icon name'''
+        return QPixmap.fromImage(resource.getIconByName(icon))
+
     def _initComponents(self):
         '''Init components'''
         self._closeAction.triggered.connect(self._close)
@@ -641,3 +660,102 @@ class MaemoFindToolBar(MaemoToolBar, FindToolBar):
                  + " border-color: white; border-style: solid;"
                  + " border-radius: 12px; background: white; }")
 
+
+class MaemoTeamworkHelpDialog(MaemoDialogMixin, CommonDialog):
+    '''Simple help viewer dialog.'''
+
+    def __init__(self, parent):
+        '''
+        @param parent: parent dialog
+        @type parent: QWidget
+        '''
+        CommonDialog.__init__(self, parent)
+        MaemoDialogMixin.__init__(self)
+        self.setWindowTitle(self.tr('Teamwork help'))
+        self._initComponents()
+
+    def _initComponents(self):
+        '''Create widgets and setup dialog layout'''
+        layout = QGridLayout()
+        textViewer = QTextBrowser()
+        textViewer.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
+        textViewer.setHtml(
+    '''
+    <div><h2>Teamwork Mode</h2>
+    <p>
+    HiveMind allows multiple users to edit mind map simultaneously. This feature is
+    called "Teamwork mode" and you have just opened the teamwork control panel. This
+    document briefly highlights main topics of the teamwork mode usage. You can always
+    see it later by clicking "Help" button.
+    </p>
+    <h3>Before you start teamwork</h3>
+    <p>
+    Every participant must have a Jabber (XMPP) account to participate in a collaboration
+    session. If you do not have such kind of account, there are many places on the Internet
+    where you can get one for free (for example http://www.jabber.org).
+    </p>
+    <p>
+    Warning! Current implementation of the teamwork mode does not allow to host collaboration
+    using several Jabber servers, such as gmail.com, but you can use accounts registered on
+    these services to join hosted teamwork.
+    </p>
+    <h3>Starting teamwork</h3>
+    <p>
+    In order to start collaboration, one of the participants have to share his/her mind map
+    by pressing "Host teamwork" button. Only one step is required to setup the host - enter
+    the credentials of the Jabber account in the dialog box.
+    </p>
+    <p>
+    The other participants should use "Participate in teamwork" button to take part in
+    collaboration. They should also specify their Jabber account credentials. Configuration
+    of the participant instance requires one more step: user needs to enter Jabber ID of the
+    host. This can be done either by choosing a contact from the roster (contact list) or by
+    manual input.
+    </p>
+    <h3>Connection status indication</h3>
+    <p>
+    The status of the teamwork is indicated by the teamwork button icon.
+    <ul>
+        <li>When there is no teamwork at the time, the icon is gray. </li>
+        <li>When you are connected to the Jabber server, only the middle figure in the icon
+        is filled with the blue color.</li>
+        <li>When the teamwork is active at the time, the icon is fully colored.</li>
+    </ul>
+    </p>
+    <h3>Finishing teamwork</h3>
+    <p>
+    HiveMind uses Jabber network to transmit data, so there are two levels of network
+    interaction: one with the Jabber server and another with the collaboration group.
+    You can disable any of them either by closing connection to the Jabber server
+    ("Go offline" button) or by disconnecting from the other participants ("Finish teamwork"
+    button).
+    </p>
+    <p>
+    If you click on "Go offline" button, the connection to Jabber server is terminated and
+    network interaction is over. When you click on "Finish teamwork" button collaboration
+    session ends, so another one can be quickly setup.
+    </p>
+    <p>
+    The host is responsible for sending data among the participants, so when host finishes
+    teamwork all other participants are disconnected too.
+    </p>
+    <h3>Advanced options</h3>
+    <p>
+    HiveMind provides flexible access control system, allowing to run either public or
+    private collaboration sessions. The host is responsible for setting up an appropriate
+    access level.
+    </p>
+    <p>
+    By default the teamwork is considered opened and anyone can join it. This behavior can be
+    changed in teamwork settings dialog ("Settings" button). The list of the participants and
+    their permissions can be modified in affiliation dialog ("Edit permissions" button).
+    </p>
+    </div>
+    ''')
+        layout.addWidget(textViewer, 0, 0)
+        layout.addWidget(self._buttonBox, 1, 0)
+        self.setLayout(layout)
+
+    def _accept(self):
+        '''Close help dialog'''
+        QDialog.accept(self)
