@@ -32,6 +32,7 @@ from wokkel.ping import PingHandler
 from hivemind.attribute import readable, Observable
 from idavoll.backend import BackendService
 from PyQt4.QtCore import QObject
+from PyQt4.QtGui import QMessageBox
 from hivemind.commands import StateModificationCommand, CurrentNodeCommand, ShutdownCommand, \
     AffiliationCommand
 from hivemind.network.models import AffiliationModel, RosterModel
@@ -77,6 +78,7 @@ class NetworkController(QObject):
                 self.__transport.rosterModel)
         self.__protocol = Protocol(self.__transport, self.__errorHandler, mindMapController,
                 self.__affiliationModel)
+        self.__affiliationModel.addObserver(self.__protocol)
         self.__actionBag = actionBag
         self.__affiliationDialog = gui_factory.createAffiliationDialog(self.__affiliationModel,
                 self.__transport.rosterModel)
@@ -147,7 +149,6 @@ class NetworkController(QObject):
                 self.__networkSettingsDialog.exec_)
         self.__networkDialog.accepted.connect(self._startNetwork)
         self.__clientDialog.accepted.connect(startClient)
-        self.__affiliationDialog.accepted.connect(self.__protocol.updateAffiliationsFromModel)
 
     def updateActions(self, state):
         '''
@@ -340,9 +341,9 @@ class Protocol(Observable):
         '''
         def isClientPropagationAllowed():
             '''Whether client can publish or not'''
-            if (self.__clientRole in ['outcast', 'member'] and
-                    self.__serverPublishModel == 'publishers'):
-                return False
+#            if (self.__clientRole in ['outcast', 'member'] and
+#                    self.__serverPublishModel == 'publishers'):
+#                return False
             return True
 
         if self.__oldProtocol:
@@ -439,17 +440,6 @@ class Protocol(Observable):
         self.__pingHandler = PingHandler()
         self.__pingHandler.setHandlerParent(self.__transport.xmppClient)
 
-    def updateAffiliationsFromModel(self):
-        '''Update affiliations in node'''
-        affiliations = self.__affiliationModel.affiliations
-        for entity, affiliation in affiliations.iteritems():
-            defer = self.__storage.getNode(XMPP_NODE_NAME)
-            defer.addCallback(lambda node:
-                    node.setAffiliation(jid.internJID(entity), affiliation))
-            command = AffiliationCommand(entity, affiliation)
-            self.__oldProtocol.proposeChangeset(
-                    Changeset(parser.serializeCommand(command), 'command'))
-
     def processNotification(self, *values, **kvargs):
         '''Process notification'''
         for event in kvargs.keys():
@@ -460,6 +450,15 @@ class Protocol(Observable):
         entity, affiliation = event
         self.__affiliationModel.updateAffiliation(entity, affiliation)
 
+    def _processModelAffiliationEvent(self, event):
+        '''Process affiliation event from model'''
+        entity, affiliation = event
+        defer = self.__storage.getNode(XMPP_NODE_NAME)
+        defer.addCallback(lambda node: node.setAffiliation(jid.internJID(entity), affiliation))
+        command = AffiliationCommand(entity, affiliation)
+        self.__oldProtocol.proposeChangeset(
+                Changeset(parser.serializeCommand(command), 'command'))
+
     def _processErrorEvent(self, failure):
         '''Process error event'''
         self.__errorHandler.mapErrorEvent(failure)
@@ -502,6 +501,8 @@ class Protocol(Observable):
         if not self.__isServiceRole:
             if isinstance(data, ShutdownCommand):
                 self.close()
+                QMessageBox.warning(None, self.__mindMapController.tr('Warning'),
+                        self.__mindMapController.tr('Service has been disconnected'))
                 return
             elif isinstance(data, AffiliationCommand):
                 if data.entity == self.__transport.xmppClient.jid.userhost():
