@@ -31,11 +31,12 @@ Provides functions for creating a mind map object from XML file and vice versa
 import parser #pylint: disable=W0406
 
 from lxml import etree as xml
-from PyQt4.QtCore import QFile, QIODevice, QTextStream
+from PyQt4.QtCore import QFile, QIODevice, QTextStream, QCoreApplication
 from PyQt4.QtGui import QTextDocument, QColor, QFont, QUndoStack
 from hivemind.core import Node, MindMap, Cloud, Attribute, AttributeRegistry
 from hivemind.commands import AddNodeCommand, MoveNodeCommand, RemoveNodeCommand, \
-StateModificationCommand, ActionCommand, CurrentNodeCommand, ShutdownCommand
+    StateModificationCommand, ActionCommand, CurrentNodeCommand, ShutdownCommand, \
+    AffiliationCommand, ConfigurationCommand
 
 
 def _parseNode(root, parent = None):
@@ -51,6 +52,7 @@ def _parseNode(root, parent = None):
     @return: root map object
     @rtype: Node
     '''
+    QCoreApplication.instance().processEvents()
     storeAttrs = {}
     attributes = _castAttributes(root.attrib)
     # prevent '__text' field creation
@@ -583,6 +585,36 @@ def _fillShutdownCommandElement(command, commandElement):
     '''
     commandElement.append(xml.Element('reason', reason = command.reason))
 
+def _fillAffiliationCommandElement(command, commandElement):
+    '''
+    Add required content to string XML element
+
+    @param command: command object
+    @type command: AffiliationCommand
+
+    @param commandElement: XML element for processing
+    @type commandElement: lxml Element
+    '''
+    params = xml.Element('affiliation_params')
+    params.set('entity', command.entity)
+    params.set('affiliation', command.affiliation)
+    commandElement.append(params)
+
+def _fillConfigurationCommandElement(command, commandElement):
+    '''
+    Add required content to string XML element
+
+    @param command: command object
+    @type command: ConfigurationCommand
+
+    @param commandElement: XML element for processing
+    @type commandElement: lxml Element
+    '''
+    params = xml.Element('configuration')
+    for option, value in command.config.iteritems():
+        params.set(option, value)
+    commandElement.append(params)
+
 def _commandToXml(command):
     '''
     Create lxml Element from command object
@@ -718,6 +750,30 @@ def _deserializeShutdownCommand(commandElement, mindMap):
     nodeElement = commandElement[0]
     return ShutdownCommand(nodeElement.get('reason'))
 
+def _deserializeAffiliationCommand(commandElement, mindMap):
+    '''
+    Deserialize AffiliationCommand command from XML string
+
+    @rtype: AffiliationCommand
+    '''
+    paramsElement = commandElement[0]
+    entity = paramsElement.get('entity')
+    affiliation = paramsElement.get('affiliation')
+    return AffiliationCommand(entity, affiliation)
+
+def _deserializeConfigurationCommand(commandElement, mindMap):
+    '''
+    Deserialize ConfigurationCommand command from XML string
+
+    @rtype: AffiliationCommand
+    '''
+    paramsElement = commandElement[0]
+    command = ConfigurationCommand()
+    for option in paramsElement.items():
+        name, value = option
+        command.setOption(name, value)
+    return command
+
 def _xmlToCommand(commandElement, mindMap):
     '''
     Create command object from lxml Element
