@@ -54,7 +54,7 @@ Font parameters names
 '''
 
 __BOOLEAN_PARAMETERS = ('confirmDelete', 'autosaveEnabled', 'readOnly', 'firstClick',
-                        'useExtendedMenu', 'presentationMode')
+                        'useExtendedMenu', 'presentationMode', 'teamworkModeFirstRun')
 '''
 Boolean parameters names
 '''
