@@ -1,6 +1,6 @@
 # -*- test-case-name: idavoll.test.test_backend -*-
 #
-# Copyright (c) 2003-2009 Ralph Meijer
+# Copyright (c) 2003-2010 Ralph Meijer
 # See LICENSE for details.
 
 """
@@ -21,10 +21,11 @@ from twisted.application import service
 from twisted.python import components, log
 from twisted.internet import defer, reactor
 from twisted.words.protocols.jabber.error import StanzaError
-from twisted.words.xish import domish, utility
+from twisted.words.xish import utility
 
-from wokkel.iwokkel import IDisco, IPubSubService
-from wokkel.pubsub import PubSubService, PubSubError
+from wokkel import disco
+from wokkel.iwokkel import IPubSubResource
+from wokkel.pubsub import PubSubResource, PubSubError
 
 from idavoll import error, iidavoll
 from idavoll.iidavoll import IBackendService, ILeafNode
@@ -63,6 +64,29 @@ class BackendService(service.Service, utility.EventDispatcher):
                      "never": "Never",
                      "on_sub": "When a new subscription is processed"}
                 },
+            "pubsub#publish_model":
+                {"type": "list-single",
+                "label": "Specify the publisher model",
+                "options": {
+                    "publishers": "Only publishers may publish",
+                    "subscribers": "Subscribers may publish",
+                    "open": "Anyone may publish"}
+                },
+            'pubsub#access_model':
+                {'type': 'list-single',
+                'label': 'Who may subscribe and retrieve items',
+                'options': {
+                    'authorize': 'Subscription requests must be '+
+                        'approved and only subscribers may retrieve ' +
+                        'items',
+                    'open': 'Anyone may subscribe and retrieve items',
+                    'presence': 'Anyone with a presence subscription ' +
+                        'of both or from  may subscribe and ' +
+                        'retrieve items',
+                    'roster': 'Anyone in the specified roster group(s)' +
+                        ' may subscribe and retrieve items',
+                    'whitelist':'Only those on a whitelist may ' +
+                        'subscribe and retrieve items'}}
             }
 
     subscriptionOptions = {
@@ -129,7 +153,14 @@ class BackendService(service.Service, utility.EventDispatcher):
 
     def _checkAuth(self, node, requestor):
         def check(affiliation, node):
-            if affiliation not in ['owner', 'publisher']:
+            configuration = node.getConfiguration()
+            if affiliation == 'outcast':
+                raise error.Forbidden()
+            if (affiliation == 'member' and
+                    configuration['pubsub#publish_model'] != 'open'):
+                raise error.Forbidden()
+            if (not node.isSubscribed(requestor)  and
+                    configuration['pubsub#publish_model'] == 'subscribers'):
                 raise error.Forbidden()
             return node
 
@@ -227,6 +258,7 @@ class BackendService(service.Service, utility.EventDispatcher):
 
         d = self.storage.getNode(nodeIdentifier)
         d.addCallback(_getAffiliation, subscriberEntity)
+        d.addCallback(self._checkAccess)
         d.addCallback(self._doSubscribe, subscriber)
         return d
 
@@ -347,6 +379,7 @@ class BackendService(service.Service, utility.EventDispatcher):
                        itemIdentifiers=None):
         d = self.storage.getNode(nodeIdentifier)
         d.addCallback(_getAffiliation, requestor)
+        d.addCallback(self._checkAccess)
         d.addCallback(self._doGetItems, maxItems, itemIdentifiers)
         return d
 
@@ -366,6 +399,15 @@ class BackendService(service.Service, utility.EventDispatcher):
             return node.getItems(maxItems)
 
 
+    def _checkAccess(self, result):
+        node, affiliation = result
+        configuration = node.getConfiguration()
+        accessModel = configuration["pubsub#access_model"]
+        if accessModel == 'whitelist' and affiliation is None:
+            raise error.NotInWhitelist
+        return result
+
+
     def retractItem(self, nodeIdentifier, itemIdentifiers, requestor):
         d = self.storage.getNode(nodeIdentifier)
         d.addCallback(_getAffiliation, requestor)
@@ -474,12 +516,33 @@ class BackendService(service.Service, utility.EventDispatcher):
 
 
 
-class PubSubServiceFromBackend(PubSubService):
+class PubSubResourceFromBackend(PubSubResource):
     """
     Adapts a backend to an xmpp publish-subscribe service.
     """
 
-    implements(IDisco)
+    features = [
+        "config-node",
+        "create-nodes",
+        "delete-any",
+        "delete-nodes",
+        "item-ids",
+        "meta-data",
+        "publish",
+        "purge-nodes",
+        "retract-items",
+        "retrieve-affiliations",
+        "retrieve-default",
+        "retrieve-items",
+        "retrieve-subscriptions",
+        "subscribe"
+    ]
+
+    discoIdentity = disco.DiscoIdentity('pubsub',
+                                        'service',
+                                        'Idavoll publish-subscribe service')
+
+    pubsubService = None
 
     _errorMap = {
         error.NodeNotFound: ('item-not-found', None, None),
@@ -503,51 +566,29 @@ class PubSubServiceFromBackend(PubSubService):
         error.NoPublishing: ('feature-not-implemented',
                              'unsupported',
                              'publish'),
+        error.NotInWhitelist: ('not-allowed', 'closed-node', None),
     }
 
     def __init__(self, backend):
-        PubSubService.__init__(self)
+        PubSubResource.__init__(self)
 
         self.backend = backend
         self.hideNodes = False
 
-        self.pubSubFeatures = self._getPubSubFeatures()
-
         self.backend.registerNotifier(self._notify)
         self.backend.registerPreDelete(self._preDelete)
 
-
-    def _getPubSubFeatures(self):
-        features = [
-            "config-node",
-            "create-nodes",
-            "delete-any",
-            "delete-nodes",
-            "item-ids",
-            "meta-data",
-            "publish",
-            "purge-nodes",
-            "retract-items",
-            "retrieve-affiliations",
-            "retrieve-default",
-            "retrieve-items",
-            "retrieve-subscriptions",
-            "subscribe",
-        ]
-
         if self.backend.supportsInstantNodes():
-            features.append("instant-nodes")
+            self.features.append("instant-nodes")
 
         if self.backend.supportsOutcastAffiliation():
-            features.append("outcast-affiliation")
+            self.features.append("outcast-affiliation")
 
         if self.backend.supportsPersistentItems():
-            features.append("persistent-items")
+            self.features.append("persistent-items")
 
         if self.backend.supportsPublisherAffiliation():
-            features.append("publisher-affiliation")
-
-        return features
+            self.features.append("publisher-affiliation")
 
 
     def _notify(self, data):
@@ -559,19 +600,21 @@ class PubSubServiceFromBackend(PubSubService):
             subscription = data['subscription']
             d = defer.succeed([(subscription.subscriber, [subscription],
                                 items)])
-        d.addCallback(lambda notifications: self.notifyPublish(self.serviceJID,
-                                                               nodeIdentifier,
-                                                               notifications))
+        d.addCallback(lambda notifications: self.pubsubService.notifyPublish(
+                                                self.serviceJID,
+                                                nodeIdentifier,
+                                                notifications))
 
 
     def _preDelete(self, data):
         nodeIdentifier = data['nodeIdentifier']
         redirectURI = data.get('redirectURI', None)
         d = self.backend.getSubscribers(nodeIdentifier)
-        d.addCallback(lambda subscribers: self.notifyDelete(self.serviceJID,
-                                                            nodeIdentifier,
-                                                            subscribers,
-                                                            redirectURI))
+        d.addCallback(lambda subscribers: self.pubsubService.notifyDelete(
+                                                self.serviceJID,
+                                                nodeIdentifier,
+                                                subscribers,
+                                                redirectURI))
         return d
 
 
@@ -589,7 +632,7 @@ class PubSubServiceFromBackend(PubSubService):
         raise exc
 
 
-    def getNodeInfo(self, requestor, service, nodeIdentifier):
+    def getInfo(self, requestor, service, nodeIdentifier):
         info = {}
 
         def saveType(result):
@@ -614,85 +657,97 @@ class PubSubServiceFromBackend(PubSubService):
         return d
 
 
-    def getNodes(self, requestor, service):
+    def getNodes(self, requestor, service, nodeIdentifier):
         if service.resource:
             return defer.succeed([])
         d = self.backend.getNodes()
         return d.addErrback(self._mapErrors)
 
 
-    def publish(self, requestor, service, nodeIdentifier, items):
-        d = self.backend.publish(nodeIdentifier, items, requestor)
-        return d.addErrback(self._mapErrors)
-
-
-    def subscribe(self, requestor, service, nodeIdentifier, subscriber):
-        d = self.backend.subscribe(nodeIdentifier, subscriber, requestor)
-        return d.addErrback(self._mapErrors)
-
-
-    def unsubscribe(self, requestor, service, nodeIdentifier, subscriber):
-        d = self.backend.unsubscribe(nodeIdentifier, subscriber, requestor)
-        return d.addErrback(self._mapErrors)
-
-
-    def subscriptions(self, requestor, service):
-        d = self.backend.getSubscriptions(requestor)
-        return d.addErrback(self._mapErrors)
-
-
-    def affiliations(self, requestor, service):
-        d = self.backend.getAffiliations(requestor)
-        return d.addErrback(self._mapErrors)
-
-
-    def create(self, requestor, service, nodeIdentifier):
-        d = self.backend.createNode(nodeIdentifier, requestor)
-        return d.addErrback(self._mapErrors)
-
-
     def getConfigurationOptions(self):
         return self.backend.nodeOptions
 
 
-    def getDefaultConfiguration(self, requestor, service, nodeType):
-        d = self.backend.getDefaultConfiguration(nodeType)
+    def publish(self, request):
+        d = self.backend.publish(request.nodeIdentifier,
+                                 request.items,
+                                 request.sender)
         return d.addErrback(self._mapErrors)
 
 
-    def getConfiguration(self, requestor, service, nodeIdentifier):
-        d = self.backend.getNodeConfiguration(nodeIdentifier)
+    def subscribe(self, request):
+        d = self.backend.subscribe(request.nodeIdentifier,
+                                   request.subscriber,
+                                   request.sender)
         return d.addErrback(self._mapErrors)
 
 
-    def setConfiguration(self, requestor, service, nodeIdentifier, options):
-        d = self.backend.setNodeConfiguration(nodeIdentifier, options,
-                                                requestor)
+    def unsubscribe(self, request):
+        d = self.backend.unsubscribe(request.nodeIdentifier,
+                                     request.subscriber,
+                                     request.sender)
         return d.addErrback(self._mapErrors)
 
 
-    def items(self, requestor, service, nodeIdentifier, maxItems,
-                    itemIdentifiers):
-        d = self.backend.getItems(nodeIdentifier, requestor, maxItems,
-                                   itemIdentifiers)
+    def subscriptions(self, request):
+        d = self.backend.getSubscriptions(request.sender)
         return d.addErrback(self._mapErrors)
 
 
-    def retract(self, requestor, service, nodeIdentifier, itemIdentifiers):
-        d = self.backend.retractItem(nodeIdentifier, itemIdentifiers,
-                                      requestor)
+    def affiliations(self, request):
+        d = self.backend.getAffiliations(request.sender)
         return d.addErrback(self._mapErrors)
 
 
-    def purge(self, requestor, service, nodeIdentifier):
-        d = self.backend.purgeNode(nodeIdentifier, requestor)
+    def create(self, request):
+        d = self.backend.createNode(request.nodeIdentifier,
+                                    request.sender)
         return d.addErrback(self._mapErrors)
 
 
-    def delete(self, requestor, service, nodeIdentifier):
-        d = self.backend.deleteNode(nodeIdentifier, requestor)
+    def default(self, request):
+        d = self.backend.getDefaultConfiguration(request.nodeType)
         return d.addErrback(self._mapErrors)
 
-components.registerAdapter(PubSubServiceFromBackend,
+
+    def configureGet(self, request):
+        d = self.backend.getNodeConfiguration(request.nodeIdentifier)
+        return d.addErrback(self._mapErrors)
+
+
+    def configureSet(self, request):
+        d = self.backend.setNodeConfiguration(request.nodeIdentifier,
+                                              request.options,
+                                              request.sender)
+        return d.addErrback(self._mapErrors)
+
+
+    def items(self, request):
+        d = self.backend.getItems(request.nodeIdentifier,
+                                  request.sender,
+                                  request.maxItems,
+                                  request.itemIdentifiers)
+        return d.addErrback(self._mapErrors)
+
+
+    def retract(self, request):
+        d = self.backend.retractItem(request.nodeIdentifier,
+                                     request.itemIdentifiers,
+                                     request.sender)
+        return d.addErrback(self._mapErrors)
+
+
+    def purge(self, request):
+        d = self.backend.purgeNode(request.nodeIdentifier,
+                                   request.sender)
+        return d.addErrback(self._mapErrors)
+
+
+    def delete(self, request):
+        d = self.backend.deleteNode(request.nodeIdentifier,
+                                    request.sender)
+        return d.addErrback(self._mapErrors)
+
+components.registerAdapter(PubSubResourceFromBackend,
                            IBackendService,
-                           IPubSubService)
+                           IPubSubResource)
