@@ -4,7 +4,7 @@
 class Error(Exception):
     msg = ''
 
-    def __init__(self, msg = None):
+    def __init__(self, msg=None):
         self.msg = msg or self.msg
 
 
@@ -94,3 +94,9 @@ class NoPublishing(Error):
     This node does not support publishing.
     """
 
+    
+
+class NotInWhitelist(Error):
+    '''
+    Entity is not in whitelist
+    '''
